from quart import Quart, make_response, jsonify, request, g
from quart_schema import QuartSchema, RequestSchemaValidationError

from .blueprints import users_blueprint
from .db.aio import async_session
from .db.util import validate_token
from .exceptions import ExpectedError

app = Quart(__name__)
QuartSchema(app)

app.register_blueprint(users_blueprint)


@app.errorhandler(RequestSchemaValidationError)
async def handle_request_validation_error(error: RequestSchemaValidationError):
    print(error.validation_error)
    return {"error": "VALIDATION"}, 400


@app.errorhandler(ExpectedError)
async def handle_expected_error(error: ExpectedError):
    return {"code": error.type, "message": error.msg}, error.status_code


@app.before_request
async def get_user_from_token():
    """Get the current user from a token given in the `Authorization` header or the `pronounscc-token` cookie.
    If no token is set, does nothing; if an invalid token is set, raises an error."""
    token = request.headers.get("Authorization", None) or request.cookies.get(
        "pronounscc-token", None
    )
    if not token:
        return

    async with async_session() as session:
        token, user = await validate_token(session, token)
        g.token = token
        g.user = user
