from pydantic import BaseModel, Field, field_validator
from quart import Blueprint, request
from quart_schema import validate_response, validate_request

from foxnouns.db.aio import async_session
from foxnouns.db.util import user_from_ref
from foxnouns.exceptions import NotFoundError, ErrorCode
from foxnouns.models.user import UserModel, check_username
from foxnouns.settings import BASE_DOMAIN

bp = Blueprint("users_v2", __name__)


@bp.get("/api/v2/users/<user_ref>", host=BASE_DOMAIN)
@validate_response(UserModel, 200)
async def get_user(user_ref: str):
    async with async_session() as session:
        user = await user_from_ref(session, user_ref)
        if not user:
            raise NotFoundError("User not found", type=ErrorCode.UserNotFound)
        return UserModel.model_validate(user)


class EditUserRequest(BaseModel):
    username: str | None = Field(
        min_length=2, max_length=40, pattern=r"^[\w\-\.]{2,40}$", default=None
    )
    display_name: str | None = Field(min_length=2, max_length=100, default=None)

    @field_validator("username")
    @classmethod
    def check_username(cls, value):
        return check_username(value)


@bp.patch("/api/v2/users/@me", host=BASE_DOMAIN)
@validate_request(EditUserRequest)
async def edit_user(data: EditUserRequest):
    return data
